//  (C) Copyright Bryce Lelbach 2011

//  Use, modification and distribution are subject to the 
//  Boost Software License, Version 1.0. (See accompanying file 
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org for most recent version.

// PathScale EKOPath C++ Compiler

#ifndef GECODE_BOOST_COMPILER
#  define GECODE_BOOST_COMPILER "PathScale EKOPath C++ Compiler version " __PATHSCALE__
#endif

#if __PATHCC__ >= 4
#  define GECODE_BOOST_MSVC6_MEMBER_TEMPLATES
#  define GECODE_BOOST_HAS_UNISTD_H
#  define GECODE_BOOST_HAS_STDINT_H
#  define GECODE_BOOST_HAS_SIGACTION
#  define GECODE_BOOST_HAS_SCHED_YIELD
#  define GECODE_BOOST_HAS_THREADS
#  define GECODE_BOOST_HAS_PTHREADS
#  define GECODE_BOOST_HAS_PTHREAD_YIELD
#  define GECODE_BOOST_HAS_PTHREAD_MUTEXATTR_SETTYPE
#  define GECODE_BOOST_HAS_PARTIAL_STD_ALLOCATOR
#  define GECODE_BOOST_HAS_NRVO
#  define GECODE_BOOST_HAS_NL_TYPES_H
#  define GECODE_BOOST_HAS_NANOSLEEP
#  define GECODE_BOOST_HAS_LONG_LONG
#  define GECODE_BOOST_HAS_LOG1P
#  define GECODE_BOOST_HAS_GETTIMEOFDAY
#  define GECODE_BOOST_HAS_EXPM1
#  define GECODE_BOOST_HAS_DIRENT_H
#  define GECODE_BOOST_HAS_CLOCK_GETTIME
#  define GECODE_BOOST_NO_VARIADIC_TEMPLATES
#  define GECODE_BOOST_NO_UNICODE_LITERALS
#  define GECODE_BOOST_NO_TEMPLATE_ALIASES
#  define GECODE_BOOST_NO_STD_UNORDERED
#  define GECODE_BOOST_NO_STATIC_ASSERT
#  define GECODE_BOOST_NO_SFINAE_EXPR
#  define GECODE_BOOST_NO_SCOPED_ENUMS
#  define GECODE_BOOST_NO_RVALUE_REFERENCES
#  define GECODE_BOOST_NO_RAW_LITERALS
#  define GECODE_BOOST_NO_NULLPTR
#  define GECODE_BOOST_NO_NUMERIC_LIMITS_LOWEST
#  define GECODE_BOOST_NO_NOEXCEPT
#  define GECODE_BOOST_NO_LAMBDAS
#  define GECODE_BOOST_NO_INITIALIZER_LISTS
#  define GECODE_BOOST_NO_MS_INT64_NUMERIC_LIMITS
#  define GECODE_BOOST_NO_FUNCTION_TEMPLATE_DEFAULT_ARGS
#  define GECODE_BOOST_NO_EXPLICIT_CONVERSION_OPERATORS
#  define GECODE_BOOST_NO_DELETED_FUNCTIONS
#  define GECODE_BOOST_NO_DEFAULTED_FUNCTIONS
#  define GECODE_BOOST_NO_DECLTYPE
#  define GECODE_BOOST_NO_CONSTEXPR
#  define GECODE_BOOST_NO_COMPLETE_VALUE_INITIALIZATION
#  define GECODE_BOOST_NO_CHAR32_T
#  define GECODE_BOOST_NO_CHAR16_T
#  define GECODE_BOOST_NO_AUTO_MULTIDECLARATIONS
#  define GECODE_BOOST_NO_AUTO_DECLARATIONS
#  define GECODE_BOOST_NO_UNIFIED_INITIALIZATION_SYNTAX
#  define GECODE_BOOST_NO_0X_HDR_UNORDERED_SET
#  define GECODE_BOOST_NO_0X_HDR_UNORDERED_MAP
#  define GECODE_BOOST_NO_0X_HDR_TYPEINDEX
#  define GECODE_BOOST_NO_0X_HDR_TUPLE
#  define GECODE_BOOST_NO_0X_HDR_THREAD
#  define GECODE_BOOST_NO_0X_HDR_SYSTEM_ERROR
#  define GECODE_BOOST_NO_0X_HDR_REGEX
#  define GECODE_BOOST_NO_0X_HDR_RATIO
#  define GECODE_BOOST_NO_0X_HDR_RANDOM
#  define GECODE_BOOST_NO_0X_HDR_MUTEX
#  define GECODE_BOOST_NO_0X_HDR_INITIALIZER_LIST
#  define GECODE_BOOST_NO_0X_HDR_FUTURE
#  define GECODE_BOOST_NO_0X_HDR_FORWARD_LIST
#  define GECODE_BOOST_NO_0X_HDR_CONDITION_VARIABLE
#  define GECODE_BOOST_NO_0X_HDR_CODECVT
#  define GECODE_BOOST_NO_0X_HDR_CHRONO
#endif

