//  (C) Copyright John Maddock 2002. 
//  Use, modification and distribution are subject to the 
//  Boost Software License, Version 1.0. (See accompanying file 
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org for most recent version.

#define GECODE_BOOST_PLATFORM "AmigaOS"

#define GECODE_BOOST_DISABLE_THREADS
#define GECODE_BOOST_NO_CWCHAR
#define GECODE_BOOST_NO_STD_WSTRING
#define GECODE_BOOST_NO_INTRINSIC_WCHAR_T
 

