body,html {
background:#FFFFFF url(back.png) repeat-y;
font-family:Arial,Helvetica,sans-serif;
color:#000000;
margin:0;
padding:0;
}

#content {
position:absolute; top:30px; left:60px;
width:60%;
padding:0 0 50px 0;
color:#000000;
}

#contentfront {
position:absolute; top:250px; left:60px;
width:50%;
padding:0 0 50px 0;
color:#000000;
}

#logo {
position:absolute; top:20px; left:75%;
}
html>body #logo {
position:fixed;
}

#headline {
position:absolute; top:15px; left:330px;
width:40%;
}

#headlinefront {
position:absolute; top:20px; left:320px;
width:40%;
}

#footline {
position:absolute; bottom:10px; left:70%;
background:#FFFFFF;
width:26%;
border-top:1px dotted #27393E;
padding:5px 0 0 0;
z-index:4;
}
html>body #footline {
position:fixed;
}

#navigation {
position:absolute; top:120px; left:70%;
width:26%;
/*height:200px;*/
background:#FFFFFF;
padding:0px 10px 10px 0px;
/*border-left:5px solid #0b7646;*/
border-left: 1px dotted #27393E;
/*border-bottom:1px dotted #27393E;*/
/*padding:0;*/
margin:0px;
vertical-align:bottom;
z-index:4;
}
html>body #navigation {
position:fixed;
}
#navigation2 {
padding:11px 0 0 15px;
margin:0;
z-index:5;
}

#news {
position:absolute; top:200px; left:70%;
width:200px;
/*padding:0px 10px 10px 0px;*/
padding:0 19px 100px 14px;
border-left:1px dotted #27393E;
/*border-bottom:1px dotted #27393E;*/
z-index:2;
}

h3.version {
text-align: center;
}

/*
  Links
*/

a {
color:#005ba1;
text-decoration: underline;
}

a:visited {
color:#005ba1;
text-decoration: none;
}

a:hover {
background-color: #f2f2ff
}

a.el {
color:#005ba1;
text-decoration: underline;
}

a.el:visited {
color:#005ba1;
text-decoration: none;
}

a.anchor {
color:#000000;
text-decoration:none;
}

a.navigation {
font-family:Verdana,Arial,Helvetica,sans-serif;
font-size:100%;
text-decoration:none;
border-right:5px solid #0b7646;
letter-spacing:2px;
color:#0b7646;
padding:0 8px 1px 8px;
margin:4px 0 0 0;
/*width:169px;*/
display:block;
}

a.navigation2 {
font-family:Verdana,Arial,Helvetica,sans-serif;
font-size:100%;
text-decoration:none;
border-right:5px solid #000000;
letter-spacing:2px;
color:#000000;
padding:0 8px 1px 8px;
margin:4px 0 0 0;
display:block;
}

a.navigation3 {
font-family:Verdana,Arial,Helvetica,sans-serif;
font-size:100%;
text-decoration:none;
border-right:5px solid #000000;
letter-spacing:2px;
color:#555555;
padding:0 8px 1px 8px;
margin:4px 0 0 0;
display:block;
}

a.navigation:visited {
color:#0b7646;
}

a.navigation:hover {
background:#0b7646;
color:#FFFFFF;
}

a.navigation2:hover {
background:none;
}

BODY,TD {
       font-size: 90%;
}
H1 {
	text-align: center;
       font-size: 130%;
}
H2 {
       font-size: 115%;
}
H3 {
       font-size: 100%;
}
CAPTION {}

DIV.qindex {
	width: 100%;
	border: 1px solid #b0b0b0;
	text-align: center;
	margin: 2px;
	padding: 2px;
	line-height: 140%;
}
DIV.nav {
	width: 100%;
	border: 1px solid #b0b0b0;
	text-align: center;
	margin: 2px;
	padding: 2px;
	line-height: 140%;
}
DIV.navtab {
       border: 1px solid #b0b0b0;
       text-align: center;
       margin: 2px;
       margin-right: 15px;
       padding: 2px;
}
TD.navtab {
       font-size: 70%;
}
A.qindex {
       text-decoration: none;
       color: #0b7646;
}
A.qindex:visited {
       text-decoration: none;
       color:#07472a;
}
A.qindex:hover {
	text-decoration: none;
	background-color:#0b7646;
	color:#FFFFFF;
}
A.qindexHL {
	text-decoration: none;
	background-color: #6666cc;
	color: #ffffff;
	border: 1px double #9295C2;
}
A.qindexHL:hover {
	text-decoration: none;
	background-color:#0b7646;
	color:#FFFFFF;
}
A.qindexHL:visited { text-decoration: none; background-color: #6666cc; color: #ffffff }
A.el { text-decoration: none;
}
A.elRef {
}
A.code:link { text-decoration: none; font-weight: normal; color: #0000FF}
A.code:visited { text-decoration: none; font-weight: normal; color: #0000FF}
A.codeRef:link { font-weight: normal; color: #0000FF}
A.codeRef:visited { font-weight: normal; color: #0000FF}
A:hover { text-decoration: none; background-color: #f2f2ff }
DL.el { margin-left: -1cm }
.fragment {
       font-family: Fixed, monospace;
       font-size: 95%;
}
PRE.fragment {
	border: 1px solid #CCCCCC;
	background-color: #f5f5f5;
	margin-top: 4px;
	margin-bottom: 4px;
	margin-left: 2px;
	margin-right: 8px;
	padding-left: 6px;
	padding-right: 6px;
	padding-top: 4px;
	padding-bottom: 4px;
}
DIV.ah { background-color: black;
color: #ffffff; margin-bottom: 3px; margin-top: 3px }
TD.md { background-color: #F4F4FB; }
TD.mdPrefix {
       background-color: #F4F4FB;
       color: #606060; font-size: 80%;
}
TD.mdname1 { background-color: #F4F4FB; color: #602020; }
TD.mdname { background-color: #F4F4FB; color: #602020; }
DIV.groupHeader {
       margin-left: 16px;
       margin-top: 12px;
       margin-bottom: 6px;
}
DIV.groupText { margin-left: 16px; font-style: italic; font-size: 90% }
DIV.memproto {
       background-color: #EAEAEA;
       border: 1px solid #C0C0C0;
}
BODY {
	background: white;
	color: black;
	margin-right: 20px;
	margin-left: 20px;
}
TD.indexkey {
	padding-right  : 10px;
	padding-top    : 2px;
	padding-left   : 10px;
	padding-bottom : 2px;
	margin-left    : 0px;
	margin-right   : 0px;
	margin-top     : 2px;
	margin-bottom  : 2px;
	border: 1px solid #CCCCCC;
}
TD.indexvalue {
	font-style: italic;
	padding-right  : 10px;
	padding-top    : 2px;
	padding-left   : 10px;
	padding-bottom : 2px;
	margin-left    : 0px;
	margin-right   : 0px;
	margin-top     : 2px;
	margin-bottom  : 2px;
	border: 1px solid #CCCCCC;
}
TR.memlist {
   background-color: #f0f0f0; 
}
P.formulaDsp { text-align: center; }
IMG.formulaDsp { }
IMG.formulaInl { vertical-align: middle; }
SPAN.keyword       { color: #008000 }
SPAN.keywordtype   { color: #604020 }
SPAN.keywordflow   { color: #e08000 }
SPAN.comment       { color: #800000 }
SPAN.preprocessor  { color: #806020 }
SPAN.stringliteral { color: #002080 }
SPAN.charliteral   { color: #008080 }
.mdTable {
	border: 1px solid #868686;
	background-color: #F4F4FB;
}
.mdRow {
	padding: 8px 10px;
}
.mdescLeft {
       padding: 0px 8px 4px 8px;
	font-size: 80%;
	font-style: italic;
	background-color: #FAFAFA;
	border-top: 1px none #E0E0E0;
	border-right: 1px none #E0E0E0;
	border-bottom: 1px none #E0E0E0;
	border-left: 1px none #E0E0E0;
	margin: 0px;
}
.mdescRight {
       padding: 0px 8px 4px 8px;
	font-size: 80%;
	font-style: italic;
	background-color: #FAFAFA;
	border-top: 1px none #E0E0E0;
	border-right: 1px none #E0E0E0;
	border-bottom: 1px none #E0E0E0;
	border-left: 1px none #E0E0E0;
	margin: 0px;
}
.memItemLeft {
	padding: 1px 0px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: solid;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	background-color: #FAFAFA;
	font-size: 80%;
}
.memItemRight {
	padding: 1px 8px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: solid;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	background-color: #FAFAFA;
	font-size: 80%;
}
.memTemplItemLeft {
	padding: 1px 0px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	background-color: #FAFAFA;
	font-size: 80%;
}
.memTemplItemRight {
	padding: 1px 8px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
	background-color: #FAFAFA;
	font-size: 80%;
}
.memTemplParams {
	padding: 1px 0px 0px 8px;
	margin: 4px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-color: #E0E0E0;
	border-right-color: #E0E0E0;
	border-bottom-color: #E0E0E0;
	border-left-color: #E0E0E0;
	border-top-style: solid;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
        color: #606060;
	background-color: #FAFAFA;
	font-size: 80%;
}
.search     { color: #003399;
}
FORM.search {
              margin-bottom: 0px;
              margin-top: 0px;
}
INPUT.search { font-size: 75%;
               color: #000080;
               font-weight: normal;
}
TD.tiny      { font-size: 75%;
}
.dirtab { padding: 4px;
          border-collapse: collapse;
          border: 1px solid #b0b0b0;
}
TH.dirtab {}
HR { height: 1px;
     border: none;
     border-top: 1px solid black;
}
UL {
	list-style-type: square;
}
