/* Boost interval.hpp header file
 *
 * Copyright 2000 Jens Maurer
 * Copyright 2002 Hervé Brönnimann, Guillaume Melquiond, Sylvain Pion
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or
 * copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef GECODE_BOOST_NUMERIC_INTERVAL_HPP
#define GECODE_BOOST_NUMERIC_INTERVAL_HPP

#include <gecode/third-party/boost/limits.hpp>
#include <gecode/third-party/boost/numeric/interval/interval.hpp>
#include <gecode/third-party/boost/numeric/interval/policies.hpp>

#include <gecode/third-party/boost/numeric/interval/hw_rounding.hpp>
#include <gecode/third-party/boost/numeric/interval/rounded_arith.hpp>
#include <gecode/third-party/boost/numeric/interval/rounded_transc.hpp>
#include <gecode/third-party/boost/numeric/interval/constants.hpp>

#include <gecode/third-party/boost/numeric/interval/checking.hpp>
#include <gecode/third-party/boost/numeric/interval/compare.hpp>
#include <gecode/third-party/boost/numeric/interval/utility.hpp>

#include <gecode/third-party/boost/numeric/interval/arith.hpp>
#include <gecode/third-party/boost/numeric/interval/arith2.hpp>
#include <gecode/third-party/boost/numeric/interval/arith3.hpp>
#include <gecode/third-party/boost/numeric/interval/transc.hpp>

#endif // GECODE_BOOST_NUMERIC_INTERVAL_HPP
